# -*- coding: utf-8 -*-
# Copyright (c) 2008-2012 Infrae. All rights reserved.
# See also LICENSE.txt

from silva.core import conf as silvaconf
from silva.core.conf.installer import DefaultInstaller
from zope.interface import Interface

silvaconf.extension_name('silva.security.addables')
silvaconf.extension_title('Silva Security Addables')


class Installer(DefaultInstaller):
    service_id = 'service_addablespermissions'

    def install_custom(self, root):
        if self.service_id not in root.objectIds():
            factory = root.manage_addProduct['silva.security.addables']
            factory.manage_addAddablesPermissionsService(self.service_id)

    def uninstall_custom(self, root):
        if self.service_id in root.objectIds():
            root.manage_delObjects([self.service_id])


class IExtension(Interface):
    """Marker interface for our extension.
    """


install = Installer("silva.security.addables", IExtension)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2012 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import ClassSecurityInfo
from AccessControl.requestmethod import postonly
from App.class_init import InitializeClass

from Products.Silva import roleinfo
from zope.component import getUtility

from five import grok
from silva.core import conf as silvaconf
from silva.core.interfaces import ISilvaConfigurableService
from silva.core.interfaces import ISilvaLocalService, IAddableContents
from silva.core.messages.interfaces import IMessageService
from silva.core.services.base import SilvaService
from silva.translations import translate as _
from silva.core.views import views as silvaviews
from silva.ui import rest


class IAddablesPermissionsService(
    ISilvaLocalService,
    ISilvaConfigurableService):
    pass


class AddablesPermissionsService(SilvaService):
    """Service to configure which content type is addable.
    """
    grok.implements(IAddablesPermissionsService)

    default_service_identifier = 'service_addablespermissions'
    meta_type = 'Silva Addables Permissions Service'
    title = 'Silva Addables Permissions Service'

    silvaconf.icon('AddablesPermissions.gif')

    security = ClassSecurityInfo()

    manage_options = (
        {'label':'Edit', 'action':'manage_permissions'},
        ) + SilvaService.manage_options

    security.declareProtected(
        'View management screens', 'manage_editAddablesPermissions')
    @postonly
    def manage_editAddablesPermissions(self, REQUEST):
        """Edit permissions.
        """
        root = self.get_root()
        for metatype in IAddableContents(root).get_all_addables():
            permission = 'Add ' + metatype + 's'
            current = root.rolesOfPermission(permission)
            current = [r['name'] for r in current if r['selected']]
            wanted = REQUEST.form.get(metatype)
            if wanted is None:
                # metatype not in request, just ignore it.
                continue
            authorized = False
            for role in self.manageableRoles():
                if role == wanted:
                    authorized = True
                if (not authorized) and role in current:
                    current.remove(role)
                if authorized and not (role in current):
                    current.append(role)
            root.manage_permission(permission, current, 0)

    security.declareProtected(
        'View management screens', 'manageableRoles')
    def manageableRoles(self):
        """Return roles for which we manage permissions.
        """
        return roleinfo.AUTHOR_ROLES

    security.declareProtected(
        'View management screens', 'currentAddablesPermissions')
    def currentAddablesPermissions(self):
        """Return current permission settings.
        """
        root = self.get_root()
        settings = {}
        for metatype in IAddableContents(root).get_all_addables():
            permissions = root.rolesOfPermission('Add ' + metatype + 's')
            permissions = dict([(r['name'], '' != r['selected']) for r in permissions])
            winner = None
            for role in self.manageableRoles():
                if permissions[role]:
                    if winner is None:
                        winner = role
                elif not (winner is None): # Sanaty check
                    msg = 'Invalid permission settings. %s can add %s but %s cannot.'
                    raise ValueError, msg % (winner, metatype, role)

            settings[metatype] = winner
        return settings


InitializeClass(AddablesPermissionsService)


class ManagePermissions(silvaviews.ZMIView):
    grok.context(AddablesPermissionsService)
    grok.name('manage_permissions')
    grok.require('zope2.ViewManagementScreens')

    def update(self):
        super(ManagePermissions, self).update()
        self.status = None
        if 'update' in self.request:
            self.context.manage_editAddablesPermissions(self.request)
            self.status = _("Permissions updated.")


class ConfigurePermissions(rest.FormWithTemplateREST):
    grok.adapts(rest.Screen, AddablesPermissionsService)
    grok.name('admin')
    grok.require('zope2.ViewManagementScreens')

    def get_menu_title(self):
        return _('Addables permissions configuration')

    def update(self):
        super(ConfigurePermissions, self).update()
        if 'update' in self.request:
            self.context.manage_editAddablesPermissions(self.request)
            service = getUtility(IMessageService)
            service.send(_(u"Permissions updated."),
                         self.request,
                         namespace='feedback')



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2012 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Acquisition import aq_base
from Products.Silva.testing import SilvaLayer
from zope.publisher.browser import TestRequest
import silva.security.addables

class SilvaSecurityAddablesLayer(SilvaLayer):
    default_packages = SilvaLayer.default_packages + [
        'silva.security.addables',
        ]

FunctionalLayer = SilvaSecurityAddablesLayer(silva.security.addables)


class AddablesPermissionsTestCase(unittest.TestCase):
    """Tests for silva.security.addables.
    """
    layer = FunctionalLayer

    def setUp(self):
        """After set up, install the extension.
        """
        self.root = self.layer.get_application()
        factory = self.root.manage_addProduct['silva.security.addables']
        factory.manage_addAddablesPermissionsService()

    def test_service(self):
        """Install should provide a new service.
        """
        self.assertTrue(
            hasattr(aq_base(self.root), 'service_addablespermissions'))

    def test_retrieve(self):
        """There is utilities which retrieve current settings.
        """
        service = self.root.service_addablespermissions

        expected_roles = ('Author', 'Editor', 'ChiefEditor', 'Manager')
        self.assertEqual(service.manageableRoles(), expected_roles)

        expected_perms = {
            'Mockup Asset': 'Author',
            'Mockup Non Publishable': 'Author',
            'Mockup VersionedContent': 'Author',
            'Silva AutoTOC': 'Author',
            'Silva Link': 'Author',
            'Silva Folder': 'Author',
            'Silva Image': 'Author',
            'Silva Publication': 'Editor',
            'Silva File': 'Author',
            'Silva Ghost Asset': 'Author',
            'Silva Ghost Folder': 'Editor',
            'Silva Indexer': 'Editor',
            'Silva Ghost': 'Author'}

        self.assertEqual(service.currentAddablesPermissions(), expected_perms)

    def test_modification(self):
        """Modification should work.
        """
        service = self.root.service_addablespermissions

        new_perms = {
            'Mockup Asset': 'Author',
            'Mockup Non Publishable': 'Editor',
            'Mockup VersionedContent': 'Author',
            'Silva AutoTOC': 'Manager',
            'Silva Link': 'Author',
            'Silva Folder': 'Author',
            'Silva Image': 'Author',
            'Silva Publication': 'Editor',
            'Silva File': 'Author',
            'Silva Ghost Asset': 'Author',
            'Silva Ghost Folder': 'Editor',
            'Silva Indexer': 'ChiefEditor',
            'Silva Ghost': 'Author'}

        request = TestRequest(form=new_perms, REQUEST_METHOD='POST')
        service.manage_editAddablesPermissions(request)

        self.assertEqual(service.currentAddablesPermissions(), new_perms)

    def test_corrupt_permissions(self):
        """When permission settings doesn't match Silva logic,
        should get an value error.
        """
        service = self.root.service_addablespermissions

        # Set bad permissions
        self.root.manage_permission('Add Silva Files', ('Author', 'Manager'), 0)
        self.assertRaises(ValueError, service.currentAddablesPermissions)


def test_suite():
    # Run tests
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AddablesPermissionsTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2012 Infrae. All rights reserved.
# See also LICENSE.txt

from setuptools import setup, find_packages
import os

version = '3.0.2'

tests_require = [
    'Products.Silva [test]',
    ]

setup(name='silva.security.addables',
      version=version,
      description="Configure which content type is addable for which Silva roles",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: BSD License",
        ],
      keywords='silva roles security',
      author='Sylvain Viollon',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.security.addables',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['silva', 'silva.security'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'Products.Silva',
        'five.grok',
        'setuptools',
        'silva.core.conf',
        'silva.core.interfaces',
        'silva.core.services',
        ],
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )
